package org.drools.persistence.session;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import junit.framework.TestCase;

import org.drools.KnowledgeBase;
import org.drools.KnowledgeBaseFactory;
import org.drools.base.MapGlobalResolver;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderError;
import org.drools.builder.KnowledgeBuilderErrors;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.persistence.jpa.JPAKnowledgeService;
import org.drools.runtime.Environment;
import org.drools.runtime.EnvironmentName;
import org.drools.runtime.StatefulKnowledgeSession;
import org.drools.runtime.process.WorkItem;
import org.drools.runtime.process.WorkItemHandler;
import org.drools.runtime.process.WorkItemManager;

import bitronix.tm.TransactionManagerServices;
import bitronix.tm.resource.jdbc.PoolingDataSource;

public class TimerTest extends TestCase {
	private int sessionId;
	private KnowledgeBase kbase;
	
	private WorkItem workItem;

	PoolingDataSource ds1;

	@Override
    protected void setUp() throws Exception {
        ds1 = new PoolingDataSource();
        ds1.setUniqueName( "jdbc/testDS1" );
        ds1.setClassName( "org.h2.jdbcx.JdbcDataSource" );
        ds1.setMaxPoolSize( 3 );
        ds1.setAllowLocalTransactions( true );
        ds1.getDriverProperties().put( "user",
                                       "sa" );
        ds1.getDriverProperties().put( "password",
                                       "sasa" );
        ds1.getDriverProperties().put( "URL",
                                       "jdbc:h2:mem:mydb" );
        ds1.init();

    }

    @Override
    protected void tearDown() throws Exception {
        ds1.close();
    }
    
    public void testShouldStartFlowAndSuspend() throws Exception {
		kbase = readKnowledgeBase();
		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("org.drools.persistence.jpa");
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
		env.set( EnvironmentName.TRANSACTION_MANAGER,
                TransactionManagerServices.getTransactionManager() );
		env.set( EnvironmentName.GLOBALS, new MapGlobalResolver() );/*
		 * Persistence enabled - Timer does NOT work!!!
		 */
		final StatefulKnowledgeSession ksession = JPAKnowledgeService.newStatefulKnowledgeSession(kbase, null,
				env);

		/*
		 * Persistence disabled - Timer DOES work!!!
		 */
		//ksession = kbase.newStatefulKnowledgeSession();

		sessionId = ksession.getId();

		ksession.getWorkItemManager().registerWorkItemHandler(
				"doNothingHandler", new DoNothingWorkItemHandler());

		// start timer thread
		new Thread() {
			@Override
			public void run() {
				System.out.println("**** Timer thread started ****");
				ksession.fireUntilHalt();
			}
		}.start();
		
		ksession.startProcess("simple-embeded-timer-with-persistence", null);
		
		// Allow timer to run for 10s
		try {
			Thread.sleep(10000);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
		
		
		// put here bcoz TestCase.tearDown() will close the ds
		shouldResumeFlowAndComplete();
	}

	public void shouldResumeFlowAndComplete() {
		EntityManagerFactory emf = Persistence
				.createEntityManagerFactory("org.drools.persistence.jpa");
		Environment env = KnowledgeBaseFactory.newEnvironment();
		env.set(EnvironmentName.ENTITY_MANAGER_FACTORY, emf);
		env.set( EnvironmentName.TRANSACTION_MANAGER,
                TransactionManagerServices.getTransactionManager() );
		env.set( EnvironmentName.GLOBALS, new MapGlobalResolver() );
		StatefulKnowledgeSession ksession = JPAKnowledgeService
				.loadStatefulKnowledgeSession(sessionId, kbase, null, env);

		ksession.getWorkItemManager().registerWorkItemHandler(
				"doNothingHandler", new DoNothingWorkItemHandler());
		System.out.println("work item id: " + workItem.getId());
		
		ksession.getWorkItemManager().completeWorkItem(workItem.getId(), null);
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(ResourceFactory.newInputStreamResource(TimerTest.class
				.getResourceAsStream("/timer.rf")), ResourceType.DRF);
		KnowledgeBuilderErrors errors = kbuilder.getErrors();
		if (errors.size() > 0) {
			for (KnowledgeBuilderError error : errors) {
				System.err.println(error);
			}
			throw new IllegalArgumentException("Could not parse knowledge.");
		}
		KnowledgeBase kbase = KnowledgeBaseFactory.newKnowledgeBase();
		kbase.addKnowledgePackages(kbuilder.getKnowledgePackages());
		return kbase;
	}

	private class DoNothingWorkItemHandler implements WorkItemHandler {

		public void abortWorkItem(WorkItem workItem, WorkItemManager manager) {
			TimerTest.this.workItem = workItem;
		}

		public void executeWorkItem(WorkItem workItem, WorkItemManager manager) {
			TimerTest.this.workItem = workItem;
		}
	}
}
